from itertools import imap
from operator import itemgetter
import os
import sys

from opensubtitles import Downloader
from sorting import DefaultSorter, SimilaritySorter

VTYPES = ['3g2', '3gp', '3gp2', '3gpp', '60d', 'ajp', 'asf', 'asx', 'avchd', 
          'avi', 'bik', 'bix', 'box', 'cam', 'dat', 'divx', 'dmf', 'dv', 
          'dvr-ms', 'evo', 'flc', 'fli', 'flic', 'flv', 'flx', 'gvi', 'gvp', 
          'h264', 'm1v', 'm2p', 'm2ts', 'm2v', 'm4e', 'm4v', 'mjp', 'mjpeg', 
          'mjpg', 'mkv', 'moov', 'mov', 'movhd', 'movie', 'movx', 'mp4', 
          'mpe', 'mpeg', 'mpg', 'mpv', 'mpv2', 'mxf', 'nsv', 'nut', 'ogg', 
          'ogm', 'omf', 'ps', 'qt', 'ram', 'rm', 'rmvb', 'swf', 'ts', 'vfw', 
          'vid', 'video', 'viv', 'vivo', 'vob', 'vro', 'wm', 'wmv', 'wmx', 
          'wrap', 'wvx', 'wx', 'x264', 'xvid']

def main(argv=None):
    if argv is None:
        argv = sys.argv
    
    spath = argv[1]
    langs = argv[2]
    
    files = [os.path.join(p, f) for p,fl in 
             imap(itemgetter(0,2), os.walk(spath))
             for f in fl if f.split('.')[-1] in VTYPES]
    
    downloader = Downloader(langs, sorter=SimilaritySorter(langs))
    for movie_file in files:
        downloader.get_best_sub(movie_file)

if __name__ == '__main__':
    main(sys.argv)




import struct
import os

# CHKSUM CONFIG
BITS = 64
BYTES = BITS / 8

BUFSIZE = 1024 * 64

def os_hash(movie_file): 
    fmt = 'Q' * (BUFSIZE / BYTES)
    with open(movie_file, 'rb') as fd:
        w1 = struct.unpack(fmt, fd.read(BUFSIZE))
        fd.seek(-BUFSIZE, os.SEEK_END)
        w2 = struct.unpack(fmt, fd.read(BUFSIZE))
        return "{0:16x}".format(
            reduce(lambda a,b: (a + b) % (1 << BITS), w1 + w2) + fd.tell())

if __name__ == '__main__':
    import sys
    print "%x" % hash(sys.argv[1])



from collections import Counter
import logging

def find(list, value):
    try: 
        return list.index(value)
    except ValueError:
        return None
        
class DefaultSorter(object):
    def __init__(self, langs='all', weight=1):
        print "Available languages: ", langs
        self.langs = langs.split(',')
 

    def bestfn(self, subentry):
        idx = find(self.langs, subentry['SubLanguageID'])
        value = idx if idx is not None else len(self.langs)
        return value


def _similarity(a, b):
    make_pairs = lambda l: (l[i:i+1] for i in xrange(len(l)-1))
    tc = lambda counter: sum(count for count in counter.values())
    sa = Counter(make_pairs(a))
    sb = Counter(make_pairs(b))
    return 2.0 * tc(sa & sb) / (tc(sa) + tc(sb))

class SimilaritySorter(DefaultSorter):
    def __init__(self, langs='all'):
        super(SimilaritySorter, self).__init__(langs)
        self.movie = ''

    def bestfn(self, subentry):
        value = super(SimilaritySorter, self).bestfn(subentry)
        sn = subentry['SubFileName']
        similarity = _similarity(sn[:sn.rindex('.')], self.movie)
        logging.info("{}: Similarity is {}, lang {}".format(
            subentry['SubFileName'], similarity, subentry['SubLanguageID']))
        return 1.1 * value + 1 - similarity



from exceptions import StandardError
import gzip
import logging
import os
from subprocess import Popen, PIPE
import urllib2
import xmlrpclib
import zlib

import chksum
from sorting import DefaultSorter

USER_AGENT = 'OS Test User Agent'
XMLRPC_URI = 'http://api.opensubtitles.org/xml-rpc'
STYPES = ['aqt', 'jss', 'sub', 'ttxt', 'pjs', 'psb', 'rt', 'smi', 'ssf', 
          'srt', 'gsub', 'ssa', 'ass', 'usf', 'stl']

class CredentialsError(StandardError):
    def __init__(self, value):
        self.value = value
    
    def __str__(self):
        return repr(self.value)


class Downloader():
    def __init__(self, langs='all', sorter=None, cr_user='', cr_pass='',
                 os_language='en'):
        self.langs = langs
        self.cr_user = cr_user
        self.cr_pass = cr_pass
        self.os_language = os_language
        
        self.sorter = sorter if sorter is not None else DefaultSorter(langs)
        
        self.logged_in = False
        logging.basicConfig(level=logging.INFO)
        
    def __log_in__(self):
        self.server = xmlrpclib.ServerProxy(XMLRPC_URI)
        login = self.server.LogIn(self.cr_user, self.cr_pass, 
                                  self.os_language, USER_AGENT)
        
        if int(login['status'].split(' ')[0]) / 100 != 2:
            error("Couldn't log in")
            raise CredentialsError("Couldn't log in")
        
        self.token = login['token']
        logging.info("Logged in with token {}".format(self.token))
        self.logged_in = True        

    def require_login(f):
        def wrapper(self, *args, **kwargs):
            if not self.logged_in:
                self.__log_in__()
            return f(self, *args, **kwargs)
        return wrapper

    @require_login
    def get_subs(self, movie_file):
        file_name = os.path.basename(movie_file)
        movie_hash = chksum.os_hash(movie_file) 
        size = str(os.path.getsize(movie_file))
        
        search_query = {'sublanguageid': self.langs,
                        'moviehash': movie_hash,
                        'moviebytesize': size}

        result = self.server.SearchSubtitles(self.token, [search_query])
        if result['data'] == 'False' or not result['data']:
            logging.info("No data found for file {}".format(file_name))
            return []
         
        logging.info("Found {} results".format(len(result['data'])))
        return result['data']

    def get_best_sub(self, movie_file):
        file_root, movie_ext = os.path.splitext(movie_file)

        if any(map(os.path.exists, 
                   (file_root + '.' + subext for subext in STYPES))):
             logging.warning("Subtitle exists for movie {}".format(
                 os.path.basename(movie_file)))
             print "Subtitle exists for movie {}".format(
                 os.path.basename(movie_file))

             return None

        results = self.get_subs(movie_file)
        if not results:
            return None

        self.sorter.movie, _ = os.path.splitext(os.path.basename(movie_file))
        best = min(results, key=self.sorter.bestfn)
        
        print "Found subtitle in {lang} for movie {mn}:\n {link} => {d}"\
             .format(lang=best['LanguageName'], mn=best['MovieName'],
                     link=best['SubDownloadLink'], 
                     d=file_root + '.' + best['SubFormat'])

        file_name = file_root + '.' + best['SubFormat'] 

        with open(file_name, 'w') as subfile:
            gzsub = urllib2.urlopen(best['SubDownloadLink'])
            subfile.write(zlib.decompress(gzsub.read(), 16+zlib.MAX_WBITS))
        return file_name





from setuptools import setup

setup(name='framboise',
      version='0.1',
      description='Utility to download subtitles from opensubtitles.org',
      url='https://github.com/luisguilherme/framboise',
      author='Luis Guilherme Pereira',
      author_email='dev@luisguilherme.net',
      license='MIT',
      packages=['framboise'],
      zip_safe=True,
      entry_points = {
          'console_scripts': [ 'framboise=framboise.framboise:main' ],
          }
)
